<?php

/**
 *
 * ZPanel - A Cross-Platform Open-Source Web Hosting Control panel.
 * 
 * @package ZPanel
 * @version $Id$
 * @author JK - info@jkmods.tk
 * @copyright (c) 2013
 * @license http://opensource.org/licenses/gpl-3.0.html GNU Public License v3
 *
 * This program (ZPanel theme_manager_x) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 **/
 
class module_controller {

    static $error;
    static $ok;
    static $ok_del;
    static $ok_del_act;
    static $error_message;

    static function getModuleName() {
        $module_name = ui_language::translate(ui_module::GetModuleName());
        return $module_name;
    }

    static function getModuleIcon() {
        global $controller;
        $module_icon = "modules/" . $controller->GetControllerRequest('URL', 'module') . "/assets/icon.png";
        return $module_icon;
    }

    static function ExectuteUpdateTheme($uid, $theme) {
        global $zdbh;
        $sql = $zdbh->prepare("
            UPDATE x_accounts
            SET ac_usertheme_vc = :theme
            WHERE ac_reseller_fk = :uid
            OR ac_id_pk = :uid2");
        $sql->bindParam(':theme', $theme);
        $sql->bindParam(':uid', $uid);
        $sql->bindParam(':uid2', $uid);
        $sql->execute();
        return true;
    }

    static function ExectuteUpdateCSS($uid, $css) {
        global $zdbh;
        $sql = $zdbh->prepare("
            UPDATE x_accounts
            SET ac_usercss_vc = :css
            WHERE ac_reseller_fk = :uid
            OR ac_id_pk = :uid2");
        $sql->bindParam(':css', $css);
        $sql->bindParam(':uid', $uid);
        $sql->bindParam(':uid2', $uid);
        $sql->execute();
        return true;
    }

    static function ExecuteShowCurrentTheme($uid) {
        return ui_template::GetUserTemplate();
    }

    static function ExecuteShowCurrentCSS($uid) {
        global $zdbh;
        $numrows = $zdbh->prepare("SELECT ac_usercss_vc FROM x_accounts WHERE ac_id_pk = :uid");
        $numrows->bindParam(':uid', $uid);
        $numrows->execute();
        $result = $numrows->fetch();
        if ($result) {
            return $result['ac_usercss_vc'];
        } else {
            return false;
        }
    }

    static function ExecuteStylesList() {
        return ui_template::ListAvaliableTemplates();
    }

    static function ExecuteCSSList() {
        $currentuser = ctrl_users::GetUserDetail();
        return ui_template::ListAvaliableCSS(self::ExecuteShowCurrentTheme($currentuser['userid']));
    }

    static function getCurrentTheme() {
        global $controller;
        $currentuser = ctrl_users::GetUserDetail();
        return self::ExecuteShowCurrentTheme($currentuser['userid']);
    }

    static function getCurrentCSS() {
        global $controller;
        $currentuser = ctrl_users::GetUserDetail();
        return self::ExecuteShowCurrentCSS($currentuser['userid']);
    }

    static function getSelectThemeMenu() {
        $html = "";
        foreach (self::ExecuteStylesList() as $theme) {
            if ($theme['name'] != self::getCurrentTheme()) {
                $html .="<option value = \"" . $theme['name'] . "\">" . $theme['name'] . "</option>\n";
            } else {
                $html .="<option value = \"" . $theme['name'] . "\" selected=\"selected\">" . $theme['name'] . "</option>\n";
            }
        }
        return $html;
    }

    static function getSelectCSSMenu() {
        $html = "";
        foreach (self::ExecuteCSSList() as $css) {
            if ($css['name'] != self::getCurrentCSS()) {
                $html .="<option value = \"" . $css['name'] . "\">" . $css['name'] . "</option>\n";
            } else {
                $html .="<option value = \"" . $css['name'] . "\" selected=\"selected\">" . $css['name'] . "</option>\n";
            }
        }
        return $html;
    }

    static function getIsSelectCSS() {
        global $controller;
        $getvars = $controller->GetAllControllerRequests('URL');
        if (isset($getvars['selectcss']))
            return true;
        return false;
    }

    static function doSaveTheme() {
        global $controller;
        runtime_csfr::Protect();
        $currentuser = ctrl_users::GetUserDetail();
        $formvars = $controller->GetAllControllerRequests('FORM');
        self::ExecuteUpdateTheme($currentuser['userid'], $formvars['inTheme']);
        if (count(self::ExecuteCSSList($formvars['inTheme'])) > 1) {
            header("location: ./?module=" . $controller->GetCurrentModule() . "&selectcss=true");
        } else {
            self::ExecuteUpdateCSS($currentuser['userid'], "");
            header("location: ./?module=" . $controller->GetCurrentModule() . "&saved=true");
        }
        exit;
    }

    static function doSaveCSS() {
        global $controller;
        runtime_csfr::Protect();
        $currentuser = ctrl_users::GetUserDetail();
        $formvars = $controller->GetAllControllerRequests('FORM');
        self::ExecuteUpdateCSS($currentuser['userid'], $formvars['inCSS']);
        header("location: ./?module=" . $controller->GetCurrentModule() . "&saved=true");
        exit;
    }

    static function getResult() {
        global $controller;
        $currentuser = ctrl_users::GetUserDetail();
        if (!fs_director::CheckForEmptyValue(self::$error_message)) {
            return ui_sysmessage::shout(ui_language::translate(self::$error_message), 'zannounceerror', 'zannounce');
        }
        if (!fs_director::CheckForEmptyValue(self::$ok)) {
            return ui_sysmessage::shout(ui_language::translate("Your theme have been installed successfully!"));
        }
        if (!fs_director::CheckForEmptyValue(self::$ok_del)) {
            return ui_sysmessage::shout(ui_language::translate("Your theme have been uninstalled successfully!"));
        }
        if (!fs_director::CheckForEmptyValue(self::$ok_del_act)) {
            return ui_sysmessage::shout(ui_language::translate("Your theme have been uninstalled successfully, new theme has been updated in affected account/s!"));
        }
        
        $urlvars = $controller->GetAllControllerRequests('URL');
        if (isset($urlvars['saved'])) {
            return ui_sysmessage::shout(ui_language::translate("Your theme configuration has been saved and has been updated for all clients!"), "zannounceok");
        }
        if (isset($urlvars['selectcss'])) {
            return ui_sysmessage::shout(ui_language::translate("This theme has more than one variation, please choose a variation you'd like to use.."), "zannounceerror");
        }
        return false;
    }


    static function doInstallTheme() {
        self::$error_message = "";
        self::$error = false;
        if ($_FILES['themefile']['error'] > 0) {
            self::$error_message = "Couldn't upload the file, " . $_FILES['themefile']['error'] . "";
        } else {
            $archive_ext = fs_director::GetFileExtension($_FILES['themefile']['name']);
            $theme_folder = fs_director::GetFileNameNoExtentsion($_FILES['themefile']['name']);
            if (!fs_director::CheckFolderExists(ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder)) {
                if ($archive_ext != 'zpt') {
                    self::$error_message = "Theme type was not detected as a .zpt (ZPanel Theme) archive.";
                } else {
                    if (fs_director::CreateDirectory(ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder)) {
                        if (sys_archive::Unzip($_FILES['themefile']['tmp_name'], ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder . '/')) {
                            self::$ok = true;
                        } else {
                            self::$error_message = "Couldn't unzip the archive (" . $_FILES['themefile']['tmp_name'] . ") to " . ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder . '/';
                        }
                    } else {
                        self::$error_message = "Couldn't create theme folder in " . ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder . "";
                    }
                }
            } else {
                self::$error_message = "The theme " . $theme_folder . " is already installed on this server!";
            }
        }
        return;
    }


      /**
       * Return total of users with activated theme
       */
      static function getActiveTheme() {
       global $zdbh;
       global $controller;
       $act_theme = $controller->GetControllerRequest('URL', 'theme');
        $sql = "SELECT * FROM x_accounts WHERE ac_usertheme_vc='".$act_theme."'";
            $list  = $zdbh->query($sql);
            $list->setFetchMode(PDO::FETCH_ASSOC);
            $row   = $list->fetch();
            $total= $zdbh->query($sql)->rowCount();
            return $total;
      }


      /**
       * Return total of users with activated theme
       */
      static function getActTheme($this_theme) {
       global $zdbh;
        $sql = "SELECT * FROM x_accounts WHERE ac_usertheme_vc='".$this_theme."'";
            $list  = $zdbh->query($sql);
            $list->setFetchMode(PDO::FETCH_ASSOC);
            $row   = $list->fetch();
            $total= $zdbh->query($sql)->rowCount();
            return $total;
      }
      
        /**
         * Deletes a directory and all files and folders under it
         * Copied from http://us2.php.net/rmdir
        **/ 
       
        static function xdelAll($directory, $empty = false) {
            if(substr($directory,-1) == "/") {
                $directory = substr($directory,0,-1);
            }

            if(!file_exists($directory) || !is_dir($directory)) {
                return false;
            } elseif(!is_readable($directory)) {
                return false;
            } else {
                $directoryHandle = opendir($directory);
               
                while ($contents = readdir($directoryHandle)) {
                    if($contents != '.' && $contents != '..') {
                        $path = $directory . "/" . $contents;
                       
                        if(is_dir($path)) {
                            self::xdelAll($path);
                        } else {
                            unlink($path);
                        }
                    }
                }
               
                closedir($directoryHandle);

                if($empty == false) {
                    if(!rmdir($directory)) {
                        return false;
                    }
                }
               
                return true;
            }
        } 
   


    static function getShowAvaliableTemplates() {
        $allstyles = '';
      //  $allstyles.= '<tr><td><b>Name</b></td><td style="text-align:center;"><b>Action</b></td><td style="text-align:center;"><b>User/s</b></td></tr>';   
        $allstyles.= '<tr><td><b>Name</b></td><td style="width:100px"></td><td style="text-align:center;"><b>User/s</b></td></tr>';   
        $handle = @opendir(ctrl_options::GetSystemOption('zpanel_root') . "etc/styles");
        $chkdir = ctrl_options::GetSystemOption('zpanel_root') . "etc/styles/";
        if ($handle) {
            while ($file = readdir($handle)) {
                if ($file != "." && $file != "..") {
                    if (is_dir($chkdir . $file)) {
                    
                      $allstyles.= '<tr><td>'.$file.'</td><td></td>';
                     /* $allstyles.='<td><form action="./?module=theme_manager_x&action=ConfirmDeleteTheme" method="post" enctype="multipart/form-data">
                         <input type="hidden" name="themefiled" id="themefiled" value='.$file.' />
                         <button class="fg-button ui-state-default ui-corner-all" type="submit" title="Delete theme '.$file.'" name="submit" id="submit"><: Delete :></button>
                         <@ CSFR_Tag @>
                         </form></td>'; */
                      
                      $curr_th = self::getActTheme($file);
                         if ($curr_th == 0) {

                         } else {
                         $allstyles.= '<td style="text-align:center; color:green">Active ('.$curr_th.')</td>';
                         }
                         $allstyles.='</tr>';
                     
                    }
                }
            }
            closedir($handle);
        }
        return $allstyles;
    }


    static function getShowTemplatesDropDown() {

        global $controller;
        $curr_th = $controller->GetControllerRequest('URL', 'theme');
        $allstyles = '';  
        $handle = @opendir(ctrl_options::GetSystemOption('zpanel_root') . "etc/styles");
        $chkdir = ctrl_options::GetSystemOption('zpanel_root') . "etc/styles/";

        if ($handle) {
            while ($file = readdir($handle)) {
                if ($file != "." && $file != "..") {
                    if (is_dir($chkdir . $file)) {
                    //if ($curr_th == $file) {
                    //} else {
                      $allstyles.= ' <option value="'.$file.'">'.$file.'</option>';
                    //}   
            }
          }
        }
      }
     return $allstyles;       
    } 



    static function getShowTemplatesNoActive() {

        global $controller;
        $curr_th = $controller->GetControllerRequest('URL', 'theme');
        $allstyles = '';  
        $handle = @opendir(ctrl_options::GetSystemOption('zpanel_root') . "etc/styles");
        $chkdir = ctrl_options::GetSystemOption('zpanel_root') . "etc/styles/";

        if ($handle) {
            while ($file = readdir($handle)) {
                if ($file != "." && $file != "..") {
                    if (is_dir($chkdir . $file)) {
                    if ($curr_th == $file) {
                    } else {
                      $allstyles.= ' <option value="'.$file.'">'.$file.'</option>';
                    }   
            }
          }
        }
      }
     return $allstyles;       
    }                  



    static function doConfirmDeleteTheme() {
        global $controller;
        runtime_csfr::Protect();
        $formvars = $controller->GetAllControllerRequests('FORM');
            if (isset($_POST['themefiled'])) {
                $curr_th = self::getActTheme($_POST['themefiled']);
                if ($curr_th > 0) {
                header("location: ./?module=" . $controller->GetCurrentModule() . "&show=DeleteAct&theme=" . $_POST['themefiled'] . ""); 
                } else {
                header("location: ./?module=" . $controller->GetCurrentModule() . "&show=Delete&theme=" . $_POST['themefiled'] . "");
                }
                exit;
            }
        return false;
    }

    static function getisDeleteTheme() {
        global $controller;
        $urlvars = $controller->GetAllControllerRequests('URL');
        if ((isset($urlvars['show'])) && ($urlvars['show'] == "Delete"))
            return true;
        return false;
    }

    static function getisDeleteThemeActive() {
        global $controller;
        $urlvars = $controller->GetAllControllerRequests('URL');
        if ((isset($urlvars['show'])) && ($urlvars['show'] == "DeleteAct"))
            return true;
        return false;
    }


    static function getThemeToDel() {
        global $controller;
        if ($controller->GetControllerRequest('URL', 'theme')) {
            return $controller->GetControllerRequest('URL', 'theme');
        } else {
            return "";
        }
    }


    static function doDeleteTheme() {
        self::$error_message = "";
        self::$error = false;
        if ($_POST['ThemeDel'] == '') {
            self::$error_message = "Couldn't delete the theme, " . $_POST['ThemeDel'] . "";
        } else {
            $theme_folder = fs_director::GetFileNameNoExtentsion($_POST['ThemeDel']);
            if (!fs_director::CheckFolderExists(ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder)) {
                self::$error_message = "The theme " . $theme_folder . " does not exist on this server!";
            } else {
               $chkdir = ctrl_options::GetSystemOption('zpanel_root') . "etc/styles/";
               $rm_dir = $chkdir.$theme_folder.'/';
               self::xdelAll($rm_dir);
               self::$ok_del = true;
            }
        }
        return;
    }


    static function doDeleteThemeActivate() {
    global $zdbh;
    global $controller;
        self::$error_message = "";
        self::$error = false;

        if ($_POST['ThemeDel'] == '') {
            self::$error_message = "Couldn't delete the theme, " . $_POST['ThemeDel'] . "";
        } else {
            $theme_folder = fs_director::GetFileNameNoExtentsion($_POST['ThemeDel']);
            if (!fs_director::CheckFolderExists(ctrl_options::GetSystemOption('zpanel_root') . 'etc/styles/' . $theme_folder)) {
                self::$error_message = "The theme " . $theme_folder . " does not exist on this server!";
            } else {
            // update tables with new theme
                $th_del = $_POST['ThemeDel'];
                $th_sel = $_POST['ThemeSel'];
                $sql  = "UPDATE x_accounts SET ac_usertheme_vc='".$th_sel."' WHERE ac_usertheme_vc ='".$th_del."'";
                $zdbh->prepare($sql)->execute();
            // update tables with default css
                $sql  = "UPDATE x_accounts SET ac_usercss_vc='default' WHERE ac_usertheme_vc ='".$th_del."'";
                $zdbh->prepare($sql)->execute();            
            // remove theme from server
               $chkdir = ctrl_options::GetSystemOption('zpanel_root') . "etc/styles/";
               $rm_dir = $chkdir.$theme_folder.'/';
               self::xdelAll($rm_dir);
               self::$ok_del_act = true;
            }
        }
        return;
    }


    static function getCSFR_Tag() {
        return runtime_csfr::Token();
    }
}
?>